//------------------------------------------------------------------
//    :     basedd7.pas
//    :       DirectDraw
//------------------------------------------------------------------
unit basedd7;

interface

uses
  windows;




//------------------------------------------------------------------
//  
//------------------------------------------------------------------
function InitDirectDraw( hWnd: HWND; dwWidth, dwHeight, dwBPP: DWORD ): boolean;
procedure ReleaseDirectDraw();
function LoadData(): boolean;
procedure OnDraw( x, y, frame: DWORD );

implementation

uses
  directdraw,
  ddutils;




//------------------------------------------------------------------
// ,    
//------------------------------------------------------------------
var
  g_pDD7:            IDirectDraw7        = nil; //   DirectDraw
  g_pPrimarySurface: IDirectDrawSurface7 = nil; //  
  g_pBackBuffer:     IDirectDrawSurface7 = nil; //  

  g_pWallpaper: IDirectDrawSurface7 = nil;            //  
  g_pMovie:     array[ 0..9 ] of IDirectDrawSurface7; //  



  
//------------------------------------------------------------------
//   DirectDraw  
//------------------------------------------------------------------
function InitDirectDraw( hWnd: HWND; dwWidth, dwHeight, dwBPP: DWORD ): boolean;
var
  ddsd2: TDDSURFACEDESC2;
  ddsc2: TDDSCAPS2;
begin
  result := false;

  //    DirectDraw
  if FAILED( DirectDrawCreateEx( nil, g_pDD7, IID_IDirectDraw7, nil ) ) then
     exit;

  //   
  if FAILED( g_pDD7.SetCooperativeLevel( hWnd, DDSCL_ALLOWREBOOT or
                                               DDSCL_EXCLUSIVE or
                                               DDSCL_FULLSCREEN ) ) then
     exit;

  //      
  if FAILED( g_pDD7.SetDisplayMode( dwWidth, dwHeight, dwBPP, 0, 0 ) ) then
     exit;

  //   TDDSURFACEDESC2
  ZeroMemory( @ddsd2, SizeOf( TDDSURFACEDESC2 ) );
  ddsd2.dwSize := SizeOf( TDDSURFACEDESC2 );

  ddsd2.dwFlags := DDSD_CAPS or DDSD_BACKBUFFERCOUNT;
  ddsd2.ddsCaps.dwCaps := DDSCAPS_PRIMARYSURFACE or
                          DDSCAPS_FLIP or
                          DDSCAPS_COMPLEX;
  ddsd2.dwBackBufferCount := 1;

  //    (    )
  if FAILED( g_pDD7.CreateSurface( ddsd2, g_pPrimarySurface, nil ) ) then
     exit;

  //   TDDSCAPS2
  ZeroMemory( @ddsc2, sizeof( TDDSCAPS2 ) );
  ddsc2.dwCaps := DDSCAPS_BACKBUFFER;

  //      
  if FAILED( g_pPrimarySurface.GetAttachedSurface( ddsc2, g_pBackBuffer ) ) then
     exit;

  result := true;
end;




//------------------------------------------------------------------
//     DirectDraw
//------------------------------------------------------------------
procedure ReleaseDirectDraw();
var
  i: integer;
begin
  for i := 0 to High( g_pMovie ) do
  if g_pMovie[ i ] <> nil then
     g_pMovie[ i ] := nil;

  if g_pWallpaper <> nil then
     g_pWallpaper := nil;

  if g_pBackBuffer <> nil then
     g_pBackBuffer := nil;

  if g_pPrimarySurface <> nil then
     g_pPrimarySurface := nil;

  if g_pDD7 <> nil then
  begin
    g_pDD7.RestoreDisplayMode();
    g_pDD7 := nil;
  end;
end;




//------------------------------------------------------------------
//    bmp-   DirectDraw
//------------------------------------------------------------------
function LoadData(): boolean;
var
  pBits: pointer;
  bi:    BITMAPINFO;
  DC:    HDC;

  i: integer;
begin
  result := false;

  //     .    
  //   BITMAPINFO -      StretchDIBits()
  pBits := LoadBitMap( 'data.bmp', @bi );
  if pBits = nil then
     exit;

  //------------------------------

  //  
  CreateSurface( g_pWallpaper, 640, 480 );
  if g_pWallpaper.GetDC( DC ) = DD_OK then
  begin
    //     
    StretchDIBits( DC,
                   0, 0, 640, 480,
                   0, 64, 640, 480,
                   pBits, bi,
                   0, SRCCOPY );
    g_pWallpaper.ReleaseDC( DC );
  end;

  //   
  for i := 0 to 9 do
  begin
    CreateSurface( g_pMovie[ i ], 64, 64 ); //    
    if g_pMovie[ i ].GetDC( DC ) = DD_OK then
    begin
      StretchDIBits( DC,
                     0, 0, 64, 64,
                     64 * i, 0, 64, 64,
                     pBits, bi,
                     0, SRCCOPY );
      g_pMovie[ i ].ReleaseDC( DC );
    end;
  end;

  //   !
  pBits := nil;

  result := true;
end;




//------------------------------------------------------------------
//     
//------------------------------------------------------------------
procedure OnDraw( x, y, frame: DWORD );
begin
  g_pBackBuffer.BltFast( 0, 0, g_pWallpaper, nil, DDBLTFAST_WAIT );
  g_pBackBuffer.BltFast( x, y, g_pMovie[ frame ], nil, DDBLTFAST_WAIT );
  g_pPrimarySurface.Flip( nil, DDFLIP_WAIT );
end;

end.
