//------------------------------------------------------------------
//    :     basedd7.pas
//    :       DirectDraw
//------------------------------------------------------------------
unit basedd7;

interface

uses
  windows;




//------------------------------------------------------------------
//  
//------------------------------------------------------------------
function InitDirectDraw( hWnd: HWND; dwWidth, dwHeight, dwBPP: DWORD ): boolean;
procedure ReleaseDirectDraw();
function LoadData(): boolean;
procedure OnDraw( x, y, frame: DWORD );

implementation

uses
  directdraw,
  ddutils;




//------------------------------------------------------------------
// ,    
//------------------------------------------------------------------
var
  g_pDD7:            IDirectDraw7        = nil; //   DirectDraw
  g_pPrimarySurface: IDirectDrawSurface7 = nil; //  
  g_pBackBuffer:     IDirectDrawSurface7 = nil; //  

  g_pDataSurface: IDirectDrawSurface7 = nil;            //  
  g_pWallpaper:   IDirectDrawSurface7 = nil;            //  
  g_pMovie:       array[ 0..9 ] of IDirectDrawSurface7; //  



  
//------------------------------------------------------------------
//   DirectDraw  
//------------------------------------------------------------------
function InitDirectDraw( hWnd: HWND; dwWidth, dwHeight, dwBPP: DWORD ): boolean;
var
  ddsd2: TDDSURFACEDESC2;
  ddsc2: TDDSCAPS2;
begin
  result := false;

  //    DirectDraw
  if FAILED( DirectDrawCreateEx( nil, g_pDD7, IID_IDirectDraw7, nil ) ) then
     exit;

  //   
  if FAILED( g_pDD7.SetCooperativeLevel( hWnd, DDSCL_ALLOWREBOOT or
                                               DDSCL_EXCLUSIVE or
                                               DDSCL_FULLSCREEN ) ) then
     exit;

  //      
  if FAILED( g_pDD7.SetDisplayMode( dwWidth, dwHeight, dwBPP, 0, 0 ) ) then
     exit;

  //   TDDSURFACEDESC2
  ZeroMemory( @ddsd2, SizeOf( TDDSURFACEDESC2 ) );
  ddsd2.dwSize := SizeOf( TDDSURFACEDESC2 );

  ddsd2.dwFlags := DDSD_CAPS or DDSD_BACKBUFFERCOUNT;
  ddsd2.ddsCaps.dwCaps := DDSCAPS_PRIMARYSURFACE or
                          DDSCAPS_FLIP or
                          DDSCAPS_COMPLEX;
  ddsd2.dwBackBufferCount := 1;

  //    (    )
  if FAILED( g_pDD7.CreateSurface( ddsd2, g_pPrimarySurface, nil ) ) then
     exit;

  //   TDDSCAPS2
  ZeroMemory( @ddsc2, sizeof( TDDSCAPS2 ) );
  ddsc2.dwCaps := DDSCAPS_BACKBUFFER;

  //      
  if FAILED( g_pPrimarySurface.GetAttachedSurface( ddsc2, g_pBackBuffer ) ) then
     exit;

  result := true;
end;




//------------------------------------------------------------------
//     DirectDraw
//------------------------------------------------------------------
procedure ReleaseDirectDraw();
var
  i: integer;
begin
  for i := 0 to High( g_pMovie ) do
  if g_pMovie[ i ] <> nil then
     g_pMovie[ i ] := nil;

  if g_pWallpaper <> nil then
     g_pWallpaper := nil;

  if g_pBackBuffer <> nil then
     g_pBackBuffer := nil;

  if g_pPrimarySurface <> nil then
     g_pPrimarySurface := nil;

  if g_pDD7 <> nil then
  begin
    g_pDD7.RestoreDisplayMode();
    g_pDD7 := nil;
  end;
end;




//------------------------------------------------------------------
//    bmp-   DirectDraw
//------------------------------------------------------------------
function LoadData(): boolean;
var
  pBits:    pointer;
  bi:       BITMAPINFO;

  DC:       HDC;
  dwWidth:  DWORD;
  dwHeight: DWORD;

  rect:     TRect;
  i:        integer;
begin
  result := false;

  //     .    
  //   BITMAPINFO -      StretchDIBits()
  pBits := LoadBitMap( 'data.bmp', @bi );
  if pBits = nil then
     exit;

  //------------------------------

  dwWidth := bi.bmiHeader.biWidth;
  dwHeight := bi.bmiHeader.biHeight;

  //  ,        pBits
  if not CreateSurface( g_pDataSurface, dwWidth, dwHeight ) then
     exit;
  if g_pDataSurface.GetDC( DC ) = DD_OK then
  begin
    //     
    StretchDIBits( DC,                      // ,    
                   0, 0, dwWidth, dwHeight, //  ,  
                   0, 0, dwWidth, DwHeight, //    
                   pBits, bi,               //     BITMAPINFO
                   0, SRCCOPY );            //  
    g_pDataSurface.ReleaseDC( DC );
  end;

  //   !
  pBits := nil;

  //------------------------------

  //     data-   
  CreateSurface( g_pWallpaper, 640, 480 );
  SetRect( rect, 0, 0, 640, 480 );
  g_pWallpaper.BltFast( 0, 0, g_pDataSurface, @rect, DDBLTFAST_WAIT );

  //     data-   
  for i := 0 to 9 do
  begin
    CreateSurface( g_pMovie[ i ], 64, 64 );         //    
    SetRect( rect, 64 * i, 480, 64 * i + 64, 544 ); //     
    g_pMovie[ i ].BltFast( 0, 0, g_pDataSurface, @rect, DDBLTFAST_WAIT );
  end;

  //------------------------------

  //  data-!
  g_pDataSurface := nil;

  result := true;
end;




//------------------------------------------------------------------
//     
//------------------------------------------------------------------
procedure OnDraw( x, y, frame: DWORD );
begin
  g_pBackBuffer.BltFast( 0, 0, g_pWallpaper, nil, DDBLTFAST_WAIT );
  g_pBackBuffer.BltFast( x, y, g_pMovie[ frame ], nil, DDBLTFAST_WAIT );
  g_pPrimarySurface.Flip( nil, DDFLIP_WAIT );
end;

end.
