//------------------------------------------------------------------
//    :     ddutils.pas
//    :    DirectDraw API
//------------------------------------------------------------------
unit ddutils;

interface

uses
  windows,
  directdraw;




//------------------------------------------------------------------
//  
//------------------------------------------------------------------
function CreateSurface( var pSurface: IDirectDrawSurface7;
                        dwWidth, dwHeight: DWORD ): boolean;

function CopyBitMap( var pSurface: IDirectDrawSurface7; const hBmp: HBITMAP;
                     pWidth, pHeight: PDWORD ): boolean;

function LoadBitMap( name: pchar; pbi: PBITMAPINFO ): pointer;

implementation




//------------------------------------------------------------------
//   DirectDraw  
//------------------------------------------------------------------
function CreateSurface( var pSurface: IDirectDrawSurface7; dwWidth, dwHeight: DWORD ): boolean;
var
  ddsd2: TDDSURFACEDESC2;
  pDD7:  IDirectDraw7;
begin
  result := false;

  //     IDirectDraw7   ,
  //      
  if FAILED( DirectDrawCreateEx( nil, pDD7, IID_IDirectDraw7, nil ) ) then
     exit;

  //   . ,      
  //    DirectDraw      
  // ,     DDSCL_NORMAL
  if FAILED( pDD7.SetCooperativeLevel( 0, DDSCL_NORMAL ) ) then
     exit;

  //   ,  
  if pSurface <> nil then
     pSurface := nil;

  //   TDDSURFACEDESC2
  ZeroMemory( @ddsd2, sizeof( TDDSURFACEDESC2 ) );
  ddsd2.dwSize := sizeof( TDDSURFACEDESC2 );

  //   
  ddsd2.dwFlags        := DDSD_CAPS or DDSD_WIDTH or DDSD_HEIGHT;
  ddsd2.ddsCaps.dwCaps := DDSCAPS_OFFSCREENPLAIN;
  ddsd2.dwWidth        := dwWidth;
  ddsd2.dwHeight       := dwHeight;

  //  
  if FAILED( pDD7.CreateSurface( ddsd2, pSurface, nil ) ) then
  begin
    pDD7 := nil;
    exit;
  end;

  //    DirectDraw
  pDD7 := nil;

  result := true;
end;




//------------------------------------------------------------------
//     DirectDraw
//------------------------------------------------------------------
function CopyBitMap( var pSurface: IDirectDrawSurface7; const hBmp: HBITMAP;
                     pWidth, pHeight: PDWORD ): boolean;
var
  destDC: HDC;    // -
  srcDC:  HDC;    // -
  bm:     BITMAP; // BITMAP  windows.pas
begin
  result := false;

  //   
  srcDC := CreateCompatibleDC( 0 );
  if srcDC = 0 then
     exit;

  //    
  SelectObject( srcDC, hBmp );

  //   ,    BITMAP
  if GetObject( hBmp, sizeof( bm ), @bm ) = 0 then
     exit;

  //     
  if not CreateSurface( pSurface, bm.bmWidth, bm.bmHeight ) then
  begin
    DeleteObject( srcDC );
    exit;
  end;

  //       
  pWidth^ := bm.bmWidth;
  pHeight^ := bm.bmHeight;

  //  GDI-   
  if FAILED( pSurface.GetDC( destDC ) ) then
  begin
    DeleteObject( srcDC );
    exit;
  end;

  //     DirectDraw  GDI BitBlt()
  BitBlt( destDC, 0, 0, bm.bmWidth, bm.bmHeight, srcDC, 0, 0, SRCCOPY );

  //    
  pSurface.ReleaseDC( destDC );
  DeleteDC( srcDC );

  result := true;
end;




//------------------------------------------------------------------
//  24-      
//------------------------------------------------------------------
function LoadBitMap( name: pchar; pbi: PBITMAPINFO ): pointer;
var
  dwDIBSize:   DWORD;             //   

  bfh:         BITMAPFILEHEADER;  // 
  bi:          BITMAPINFO;        // bmp-

  hFile:       THandle;           //   
  dwBytesRead: DWORD;             //    
begin
  result := nil;

  //       
  hFile := CreateFile( name, GENERIC_READ, FILE_SHARE_READ, nil,
                       OPEN_EXISTING, 0, 0 );
  if hFile = INVALID_HANDLE_VALUE then
     exit;

  //   BITMAPFILEHEADER
  ReadFile( hFile, bfh, sizeof( BITMAPFILEHEADER ), dwBytesRead, nil );
  //    ( )
  if bfh.bfType <> $4d42 then
  begin
    CloseHandle( hFile );
    exit;
  end;

  //   BITMAPINFOHEADER
  ReadFile( hFile, bi, sizeof( BITMAPINFOHEADER ), dwBytesRead, nil );
  // ,    24-
  if bi.bmiHeader.biBitCount <> 24 then
  begin
    CloseHandle( hFile );
    exit;
  end;

  //   DIB
  dwDIBSize := GetFileSize( hFile, nil ) - sizeof( BITMAPFILEHEADER ) - sizeof( BITMAPINFOHEADER );
  //   
  result := pbyte(GlobalAllocPtr( GMEM_MOVEABLE, dwDIBSize ));
  //   DIB
  ReadFile( hFile, result^, dwDIBSize, dwBytesRead, nil );
  //  
  CloseHandle( hFile );

  //      BITMAPINFO
  if pbi <> nil then
     pbi^ := bi;
end;

end.
