//------------------------------------------------------------------
//    :     pscreen.pas
//    :       bmp-
//------------------------------------------------------------------
unit pscreen;

interface

uses
  windows,
  directdraw;




//------------------------------------------------------------------
//  
//------------------------------------------------------------------
function SaveScreenToBitMap( szFileName: pchar; const pSurface: IDirectDrawSurface7 ): boolean;

implementation




//------------------------------------------------------------------
//     DirectDraw    *.bmp
//------------------------------------------------------------------
function SaveScreenToBitMap( szFileName: pchar; const pSurface: IDirectDrawSurface7 ): boolean;
var
  hFile:          THandle;     //   
  dwBytesWritten: DWORD;       //    (  )

  fh:        BITMAPFILEHEADER; //  bmp-
  bi:        BITMAPINFOHEADER; //      

  pPixels:   pbyte;            //   
  dwPointer: DWORD;            //     

  ddsd2:     TDDSURFACEDESC2;  //       

  dwWidth:   DWORD;            // ,
  dwHeight:  DWORD;            //  
  dwBPP:     DWORD;            //    DirectDraw

  pixel:     DWORD;            //   
  r, g, b:   byte;             //    

  i, j:      integer;          //    
begin
  result := false;

  //      
  if( szFileName = nil ) or ( pSurface = nil ) then
      exit;

  //     ,     
  hFile := CreateFile( szFileName, GENERIC_WRITE, FILE_SHARE_READ, nil, CREATE_ALWAYS, 0, 0 );
  if hFile = INVALID_HANDLE_VALUE then
  begin
    CloseHandle( hFile );
    exit;
  end;

  //   TDDSURFACEDESC2
  ZeroMemory( @ddsd2, sizeof( TDDSURFACEDESC2 ) );
  ddsd2.dwSize := sizeof( TDDSURFACEDESC2 );

  //   
  pSurface.GetSurfaceDesc( ddsd2 );
  dwWidth  := ddsd2.dwWidth;
  dwHeight := ddsd2.dwHeight;
  dwBPP    := ddsd2.ddpfPixelFormat.dwRGBBitCount;

  //   TDDSURFACEDESC2  
  ZeroMemory( @ddsd2, sizeof( TDDSURFACEDESC2 ) );
  ddsd2.dwSize := sizeof( TDDSURFACEDESC2 );

  //   DirectDraw
  if( FAILED( pSurface.Lock( nil, ddsd2, DDLOCK_WAIT, 0 ) ) ) then
      exit;

  //------------------------------

  pPixels := pbyte(GlobalAllocPtr( GMEM_MOVEABLE, dwWidth * dwHeight * 3 ));
  if pPixels = nil then
     exit;

  dwPointer := 0;

  //         
  //  ,       
  for j := dwHeight - 1 downto 0 do
  for i := 0 to dwWidth - 1 do
  begin
  
    case dwBPP of
      32: // 32- 
      begin
        pixel := PDWORD(DWORD(ddsd2.lpSurface) + i * 4 + j * ddsd2.lPitch)^;

        g := ((pixel shr 8) and $000000FF);
        inc( dwPointer );
        pbyte(pointer(DWORD(pPixels) + dwPointer))^ := g;

        b := ((pixel shr 16) and $000000FF);
        inc( dwPointer );
        pbyte(pointer(DWORD(pPixels) + dwPointer))^ := b;

        r := ((pixel) and $000000FF);
        inc( dwPointer );
        pbyte(pointer(DWORD(pPixels) + dwPointer))^ := r;
      end;

      24: // 24- 
      begin
        inc( dwPointer );
        pbyte(pointer(DWORD(pPixels) + dwPointer))^ := pbyte(DWORD(ddsd2.lpSurface) + i * 3 + 1 + j * ddsd2.lPitch)^;

        inc( dwPointer );
        pbyte(pointer(DWORD(pPixels) + dwPointer))^ := pbyte(DWORD(ddsd2.lpSurface) + i * 3 + 2 + j * ddsd2.lPitch)^;

        inc( dwPointer );
        pbyte(pointer(DWORD(pPixels) + dwPointer))^ := pbyte(DWORD(ddsd2.lpSurface) + i * 3 + 0 + j * ddsd2.lPitch)^;
      end;

      16: // 16- 
      begin
        if ddsd2.ddpfPixelFormat.dwGBitMask = $07e0 then //  5-6-5 ( )
	begin
          pixel := PWORD(DWORD(ddsd2.lpSurface) + i * 2 + j * ddsd2.lPitch)^;

          r := 4 * ((pixel shr 5) and $003f);
          inc( dwPointer );
          pbyte(pointer(DWORD(pPixels) + dwPointer))^ := r;

          g := 8 * ((pixel shr 11) and $001f);
          inc( dwPointer );
          pbyte(pointer(DWORD(pPixels) + dwPointer))^ := g;

          b := 8 * ((pixel) and $001f);
          inc( dwPointer );
          pbyte(pointer(DWORD(pPixels) + dwPointer))^ := b
        end
        else //  5-5-5 ( )
	begin
          pixel := PWORD(DWORD(ddsd2.lpSurface) + i * 2 + j * ddsd2.lPitch)^;

          r := 8 * ((pixel shr 5) and $001f);
          inc( dwPointer );
          pbyte(pointer(DWORD(pPixels) + dwPointer))^ := r;

          g := 8 * ((pixel shr 10) and $001f);
          inc( dwPointer );
          pbyte(pointer(DWORD(pPixels) + dwPointer))^ := g;

          b := 8 * ((pixel) and $001f);
          inc( dwPointer );
          pbyte(pointer(DWORD(pPixels) + dwPointer))^ := b;
        end;
      end
      else exit; // case of (8   )

    end;
  end;

  //   DirectDraw
  pSurface.Unlock( nil );

  //------------------------------

  //   BITMAPFILEHEADER
  fh.bfType          := $4D42; // 'B' 'M'
  fh.bfSize          := sizeof( BITMAPINFOHEADER ) + sizeof( BITMAPFILEHEADER ) +
                        dwWidth * dwHeight * dwBPP;
  fh.bfReserved1     := 0;
  fh.bfReserved2     := 0;
  fh.bfOffBits       := sizeof( BITMAPINFOHEADER ) + sizeof( BITMAPFILEHEADER );

  //   BITMAPINFOHEADER
  bi.biSize          := sizeof( BITMAPINFOHEADER );
  bi.biWidth         := dwWidth;
  bi.biHeight        := dwHeight;
  bi.biPlanes        := 1;
  bi.biBitCount      := 24;
  bi.biCompression   := BI_RGB;
  bi.biSizeImage     := 0;
  bi.biXPelsPerMeter := 10000;
  bi.biYPelsPerMeter := 10000;
  bi.biClrUsed       := 0;
  bi.biClrImportant  := 0;

  //   
  if WriteFile( hFile, fh, sizeof( BITMAPFILEHEADER ), dwBytesWritten, nil ) = BOOL(FALSE) then
  begin
    CloseHandle( hFile );
    exit;
  end;

  //      
  if WriteFile( hFile, bi, sizeof( BITMAPINFOHEADER ), dwBytesWritten, nil ) = BOOL(FALSE) then
  begin
    CloseHandle( hFile );
    exit;
  end;

  //   
  if WriteFile( hFile, pPixels^, dwWidth * dwHeight * 3, dwBytesWritten, nil ) = BOOL(FALSE) then
  begin
    CloseHandle( hFile );
    pPixels := nil;
    exit;
  end;

  //  
  CloseHandle( hFile );
  pPixels := nil;

  result := true;
end;

end.






